@@ -28,17 +28,6 @@
 #define EXP_BLINDING_RANDOMNESS_LEN ((128 + MP_DIGIT_BIT - 1) / MP_DIGIT_BIT)
 #define EXP_BLINDING_RANDOMNESS_LEN_BYTES (EXP_BLINDING_RANDOMNESS_LEN * sizeof(mp_digit))
 
-/*
-** Number of times to attempt to generate a prime (p or q) from a random
-** seed (the seed changes for each iteration).
-*/
-#define MAX_PRIME_GEN_ATTEMPTS 10
-/*
-** Number of times to attempt to generate a key.  The primes p and q change
-** for each attempt.
-*/
-#define MAX_KEY_GEN_ATTEMPTS 10
-
 /* Blinding Parameters max cache size  */
 #define RSA_BLINDING_PARAMS_MAX_CACHE_SIZE 20
 
@@ -208,38 +197,6 @@ cleanup:
     return rv;
 }
 
-SECStatus
-generate_prime(mp_int *prime, int primeLen)
-{
-    mp_err err = MP_OKAY;
-    SECStatus rv = SECSuccess;
-    int piter;
-    unsigned char *pb = NULL;
-    pb = PORT_Alloc(primeLen);
-    if (!pb) {
-        PORT_SetError(SEC_ERROR_NO_MEMORY);
-        goto cleanup;
-    }
-    for (piter = 0; piter < MAX_PRIME_GEN_ATTEMPTS; piter++) {
-        CHECK_SEC_OK(RNG_GenerateGlobalRandomBytes(pb, primeLen));
-        pb[0] |= 0xC0;            /* set two high-order bits */
-        pb[primeLen - 1] |= 0x01; /* set low-order bit       */
-        CHECK_MPI_OK(mp_read_unsigned_octets(prime, pb, primeLen));
-        err = mpp_make_prime_secure(prime, primeLen * 8, PR_FALSE);
-        if (err != MP_NO)
-            goto cleanup;
-        /* keep going while err == MP_NO */
-    }
-cleanup:
-    if (pb)
-        PORT_ZFree(pb, primeLen);
-    if (err) {
-        MP_TO_SEC_ERROR(err);
-        rv = SECFailure;
-    }
-    return rv;
-}
-
 /*
  *  make sure the key components meet fips186 requirements.
  */
